#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, absolute_import, unicode_literals
import sys
import datetime
import numpy as np
from etasdk import *
try:
    import statsmodels.tsa.stattools as ts
except:
    print('请安装statsmodels库')
    sys.exit(-1)

'''
期货策略：跨期套利
本策略根据EG两步法(1.序列同阶单整2.OLS残差平稳)判断序列具有协整关系后(若无协整关系则全平仓位不进行操作)
通过计算两个价格序列回归残差的均值和标准差并用均值加减0.9倍标准差得到上下轨
在价差突破上轨的时候做空价差;在价差突破下轨的时候做多价差
若有仓位,在残差回归至上下轨内的时候平仓
回测数据为:rb1801和rb1805的1min数据
回测时间为:2017-09-25 到2017-10-01
撮合周期：1分钟
'''


def on_initialize(api):
    # 自定义变量，代表撮合周期
    api.bar_span = ETimeSpan.MIN_1
    # 设置bar长度
    api.data_len = 800
    # 进行套利的品种
    api.tickers_code = ['rb1801.CF', 'rb1805.CF']

    api.context.init = {
        # symbols里设置单个股票或者其他有效的标的
        # symbol_sets里设置标的集合，如此处的IF.PRD指的就是IF品种的所有标准合约，不包含IFZ0.CF和IFZ1.CF
        # 注：symbols和symbol_sets不能全部为空
        'symbol_sets': ['rb.PRD'],
        # 必填：设置所需k线类型，interval可以选取1分钟，5分钟，15分钟，30分钟，60分钟，日k，Tick级别，count为提前缓存的数目
        'prepared_bars': [
            {'interval': api.bar_span, 'count': 900}
        ],
        # 选填：设置撮合参数 interval为撮合周期，time_ranges为日内撮合时间段，将只在指定时间段以interval周期回调on_bar或on_handle_date
        'match_param': {
            'interval': api.bar_span,
        },
        'mode': 'group_only',
        # 选填：仅回测生效的项，不设置时以web页面上的设置为准
        'backtest': {
            # 回测开始日期
            'start_date': 20170701,
            # 回测结束日期
            'end_date': 20170712,
            # 设置滑点,取值为整数,标的价格浮动最小单位值的整数倍
            # 设置账户初始资金
            'cash': {
                # 设置A股资金
                'CS': 0,
                # 设置期货资金
                'CF': 1000000
            }
        },

        # 选填：仅模拟盘/实盘生效的项
        'realtime': {
            # 设置开盘前回调on_before_market_open响应事件时间，默认为web页面上设置的时间
            'pre_market_time': '19:00',
            # 设置收盘前最后一次回调on_bar或on_handle_data响应事件时间，默认为web页面上设置的时间
            'closing_time': '14:55',
            # 设置group模式下的组播，等待多少ms则强制响应，默认为5000ms
            'group_timeout': 5000,
        }
    }


def on_before_market_open(api, date_now):
    # 订阅行情
    api.context.pool.focus = api.tickers_code


def on_handle_data(api, time_now):
    # 获取两个品种的时间序列
    data_01 = api.get_bars_history(symbol=api.tickers_code[0], time_span=ETimeSpan.MIN_1, count=api.data_len + 1,
                                   price_mode=EPriceMode.FORMER)
    close_01 = data_01["close"].values

    data_02 = api.get_bars_history(symbol=api.tickers_code[1], time_span=ETimeSpan.MIN_1, count=api.data_len + 1,
                                   price_mode=EPriceMode.FORMER)
    close_02 = data_02["close"].values

    # 展示两个价格序列的协整检验的结果
    beta, c, resid, result = cointegration_test(close_01, close_02)

    # 如果返回协整检验不通过的结果则全平仓位等待
    if not result:
        print('协整检验不通过,全平所有仓位')
        symbol_positions = api.context.strategy.positions
        if symbol_positions:
            for symbol_obj in symbol_positions:
                if symbol_obj.positionSide == EPositionSide.SHORT:
                    api.target_position(symbol=symbol_obj.symbol, qty=0, side=EPositionSide.SHORT)
                    log.info("close old contract short position:%s", symbol_obj.symbol)
                elif symbol_obj.positionSide == EPositionSide.LONG:
                    api.target_position(symbol=symbol_obj.symbol, qty=0, side=EPositionSide.LONG)
                    log.info("close old contract long position:%s", symbol_obj.symbol)
        return

    # 计算残差的标准差上下轨
    mean = np.mean(resid)
    up = mean + 1.5 * np.std(resid)
    down = mean - 1.5 * np.std(resid)

    # 计算新残差
    resid_new = close_01[-1] - beta * close_02[-1] - c

    # 获取rb1801的多空仓位
    position_01_long = api.context.strategy.positions[api.tickers_code[0]].long
    position_01_short = api.context.strategy.positions[api.tickers_code[0]].short

    if not position_01_long and not position_01_short:
        if resid_new > up:
            api.target_position(symbol=api.tickers_code[0], qty=1, side=EPositionSide.SHORT)
            log.info("Open short at market price:%s", api.tickers_code[0])
            api.target_position(symbol=api.tickers_code[1], qty=1, side=EPositionSide.LONG)
            log.info("Open long at market price:%s", api.tickers_code[1])

        if resid_new < down:
            api.target_position(symbol=api.tickers_code[0], qty=1, side=EPositionSide.LONG)
            log.info("Open long at market price:%s", api.tickers_code[0])
            api.target_position(symbol=api.tickers_code[1], qty=1, side=EPositionSide.SHORT)
            log.info("Open short at market price:%s", api.tickers_code[1])

    # 价差回归时平仓
    elif position_01_short:
        if resid_new <= up:
            api.target_position(symbol=api.tickers_code[0], qty=0, side=EPositionSide.SHORT)
            api.target_position(symbol=api.tickers_code[1], qty=0, side=EPositionSide.LONG)
            log.info("Clear all positions")

        # 跌破下轨反向开仓
        if resid_new < down:
            api.target_position(symbol=api.tickers_code[0], qty=1, side=EPositionSide.LONG)
            log.info("Open long at market price:%s", api.tickers_code[0])
            api.target_position(symbol=api.tickers_code[1], qty=1, side=EPositionSide.SHORT)
            log.info("Open short at market price:%s", api.tickers_code[1])

    elif position_01_long:
        if resid_new >= down:
            api.target_position(symbol=api.tickers_code[0], qty=0, side=EPositionSide.SHORT)
            api.target_position(symbol=api.tickers_code[1], qty=0, side=EPositionSide.LONG)
            log.info("Clear all positions")
        # 突破上轨开多
        if resid_new > up:
            api.target_position(symbol=api.tickers_code[0], qty=1, side=EPositionSide.SHORT)
            log.info("Open short at market price:%s", api.tickers_code[0])
            api.target_position(symbol=api.tickers_code[1], qty=1, side=EPositionSide.LONG)
            log.info("Open long at market price:%s", api.tickers_code[1])


def on_terminate(api, exit_info):
    print("回测结束。。。")


# 协整检验的函数
def cointegration_test(series01, series02):
    urt_rb1801 = ts.adfuller(np.array(series01), 1)[1]
    urt_rb1805 = ts.adfuller(np.array(series02), 1)[1]
    # 同时平稳或不平稳则差分再次检验
    if (urt_rb1801 > 0.1 and urt_rb1805 > 0.1) or (urt_rb1801 < 0.1 and urt_rb1805 < 0.1):
        urt_diff_rb1801 = ts.adfuller(np.diff(np.array(series01)), 1)[1]
        urt_diff_rb1805 = ts.adfuller(np.diff(np.array(series02)), 1)[1]
        # 同时差分平稳进行OLS回归的残差平稳检验
        if urt_diff_rb1801 < 0.1 and urt_diff_rb1805 < 0.1:
            matrix = np.vstack([series02, np.ones(len(series02))]).T
            beta, c = np.linalg.lstsq(matrix, series01)[0]
            resid = series01 - beta * series02 - c
            if ts.adfuller(np.array(resid), 1)[1] > 0.1:
                result = 0.0
            else:
                result = 1.0
            return beta, c, resid, result

        else:
            result = 0.0
            return 0.0, 0.0, 0.0, result

    else:
        result = 0.0
        return 0.0, 0.0, 0.0, result
